;/******************************************************************************
*                                                                             *
*                                                                             *
*                           	Copyright by                                  *
*                                                                             *
*                         	  Azoteq (Pty) Ltd                                *
*                     	  Republic of South Africa                            *
*                                                                             *
*                     	   Tel: +27(0)21 863 0033                             *
*                    	  E-mail: info@azoteq.com                             *
*                                                                             *
*=============================================================================*
* @file 	    IQS62x.h									 				  *
* @brief 	    Header for the IQS62x device type for demo purposes			  *
* @author 		LC Theron - Azoteq PTY Ltd                             *
* @version 		V1.0.0                                                        *
* @date 	    23/05/2018                                                  *
*******************************************************************************/


#ifndef __IQS62x_H__
#define __IQS62x_H__

#include "Arduino.h"
#include "IQS621_Init.h"


// Define Product Numbers
#define IQS620_PRODUCT_NR     65
#define IQS621_PRODUCT_NR			70


#define IQS62x_RDY          2

// Enum to indicate if we will do a Capacitive or Inductive sensing setup
typedef enum Setup_Type {
	Capacitive = 0,
	Inductive
}Setup_Type_e;

/*
 *  Create Unions of bitfields to make reading and the IQS62x
 *  easier and more convenient
 */
// System Flags
typedef union
{
	// Bitfield for System Flags
    struct
    {
    	uint8_t NpSegmentActive     :1;
    	uint8_t Event	            :1;
    	uint8_t InAti	            :1;
    	uint8_t PowerMode           :2;
    	uint8_t None	            :2;
    	uint8_t ShowReset           :1;
    };
    uint8_t SystemFlags;
}SystemFlags_t;

// Global Events
typedef union
{
	// Bitfield for Global Events
    struct
    {
    	uint8_t Pxs_Event			:1;
    	uint8_t Sar_Event			:1;
    	uint8_t Hall_Event			:1;
    	uint8_t Metal_Detect_Flag   :1;
    	uint8_t System_Event 	    :1;
    	uint8_t None		        :1;
    	uint8_t Sar_Active          :1;
    	uint8_t Hall_Cover_Closed   :1;
    };
    uint8_t GlobalEvents;
}GlobalEvents_IQS620_t;

typedef union
{
  // Bitfield for Global Events
    struct
    {
      uint8_t Pxs_Event     :1;
      uint8_t Sar_Event     :1;
      uint8_t Hall_Event      :1;
      uint8_t Metal_Detect_Flag   :1;
      uint8_t Temp_Event        :1;
      uint8_t Sys_Event         :1;
      uint8_t Pmu_Event           :1;
      uint8_t Sar_Active        :1;
    };
    uint8_t GlobalEvents;
}GlobalEvents_IQS620n_t;

// Global Events - TODO
typedef union
{
	// Bitfield for Global Events
    struct
    {
    	uint8_t Pxs_Event			:1;
    	uint8_t Hall_Event			:1;
    	uint8_t ALS_Event			:1;
    	uint8_t Metal_Detect_Flag   :1;
    	uint8_t Temp_Event 	    	:1;
    	uint8_t System_Event        :1;
    	uint8_t PMU_Event           :1;
    };
    uint8_t GlobalEvents;
}GlobalEvents_IQS621_t;

typedef union
{
  // Bitfield for PXS UI Flags
    struct
    {
      uint8_t CH0_P_Out       :1;
      uint8_t CH1_P_Out         :1;
      uint8_t CH2_P_Out         :1;
      uint8_t None            :1;
      uint8_t CH0_T_Out           :1;
      uint8_t CH1_T_Out           :1;
      uint8_t CH2_T_Out           :1;
    };
    uint8_t PXSUIFlags;
}PXSUIFlags_IQS620n_t;

// PXS UI Flags
typedef union
{
	// Bitfield for PXS UI Flags
    struct
    {
    	uint8_t CH0_P_Out		    :1;
    	uint8_t CH1_P_Out	        :1;
    	uint8_t None		        :2;
    	uint8_t CH0_T_Out           :1;
    	uint8_t CH1_T_Out           :1;
    };
    uint8_t PXSUIFlags;
}PXSUIFlags_t;


// SAR and Metal Flags
typedef union
{
	// Bitfield for SAR and Metal Flags
    struct
    {
    	uint8_t FHalt			    :1;
    	uint8_t Movement            :1;
    	uint8_t Quick_Release       :1;
    	uint8_t None		        :1;
    	uint8_t Sar_Active	        :1;
    	uint8_t Metal_Prox	        :1;
    	uint8_t Metal_Touch	        :1;
    	uint8_t Metal_Type	        :1;
    };
    uint8_t SARMetalFlags;
}SARMetalFlags_t;

// Metal Detect Output
typedef union
{
	// Bitfield for Metal Detect Output
    struct
    {
    	uint8_t Metal_Low;
    	uint8_t Metal_High;
    };
    uint16_t MetalValue;
}MetalValue_t;

// SAR and Metal Flags
typedef union
{
	// Bitfield for SAR and Metal Flags
    struct
    {
    	uint8_t Metal_Prox	        :1;
    	uint8_t Metal_Touch	        :1;
    	uint8_t Metal_Type	        :1;
    };
    uint8_t Metal_UI_Flags;
}Metal_UI_Flags_t;

// ALS Flags
typedef union
{
	// Bitfield for PXS UI Flags
    struct
    {
    	uint8_t ALS_Value		    :4;
    	uint8_t None	            :3;
    	uint8_t Light_Dark          :1;
    };
    uint8_t ALSFlags;
}ALSFlags_t;

// ALS Output
typedef union
{
	// Bitfield for PXS UI Flags
    struct
    {
    	uint8_t ALS_Low;
    	uint8_t ALS_High;
    };
    uint16_t ALSValue;
}ALSValue_t;

// Hall UI Flags
typedef union
{
	// Bitfield for PXS UI Flags
    struct
    {
    	uint8_t Hall_N_S			:1;
    	uint8_t Hall_Prox			:1;
    	uint8_t Hall_Touch			:1;
    };
    uint8_t HallFlags;
}HallFlags_t;

// Hall Value
typedef union
{
	// Bitfield for PXS UI Flags
    struct
    {
    	uint8_t Hall_Low;
    	uint8_t Hall_High;
    };
    uint16_t HallValue;
}HallValue_t;




// Hall Degrees
typedef union
{
	// Bitfield for Degrees output
    struct
    {
    	uint8_t Hall_Degrees_Low;
    	uint8_t Hall_Degrees_High;
    };
    uint16_t HallDegrees;
}HallDegrees_t;


// Temperature Flags Value
typedef union
{
	// Bitfield for Temp Flags
    struct
    {
    	uint8_t None				:7;
    	uint8_t Temp_Trip			:1;
    };
    uint16_t TempFlags;
}TempFlags_t;

// Hall Value
typedef union
{
	// Bitfield for PXS UI Flags
    struct
    {
    	uint8_t Temp_Low;
    	uint8_t Temp_High;
    };
    uint16_t TempValue;
}TempValue_t;

// PXS CH  Value
typedef union
{
	// Bitfield for PXS UI Flags
    struct
    {
    	uint8_t Ch_Low;
    	uint8_t Ch_High;
    };
    uint16_t Ch;
}Ch_t;

// "Object" for IQS620 production
typedef struct IQS620n {

  // System Flags
  SystemFlags_t SystemFlags;

  // Global Events
  GlobalEvents_IQS620n_t GlobalEvents;

  // Proxsense UI Flags
  PXSUIFlags_IQS620n_t PXSUIFlags;

  // Sar and Metal Flags
  SARMetalFlags_t SARMetalFlags;

  // Metal Detect Value
  MetalValue_t MetalValue;

  // Hall UI Flags
  HallFlags_t HallFlags;

  // Hall Value (output)
  HallValue_t HallValue;

  // Temperature Flags
  TempFlags_t TempFlags;

  // Temperature Value
  TempValue_t TempValue;

  // Channel 0 data
  Ch_t Ch[6];

  // LTA data - Ch1
  Ch_t LTA1;

} IQS620n_t;

// "Object" for IQS621
typedef struct IQS621 {

	// System Flags
	SystemFlags_t SystemFlags;

	// Global Events
	GlobalEvents_IQS621_t GlobalEvents;

	// Proxsense UI Flags
	PXSUIFlags_t PXSUIFlags;

	// Metal UI flags
	Metal_UI_Flags_t MetalFlags;

	// Metal Detect Value
	MetalValue_t MetalValue;

	// Als Flags
	ALSFlags_t ALSFlags;

	// ALS value
	ALSValue_t ALSValue;

	// Hall UI Flags
	HallFlags_t HallFlags;

	// Hall Value (output)
	HallValue_t HallValue;

	// Temperature flags
	TempFlags_t TempFlags;

	// Temperature value
	TempValue_t TempValue;

	// Channel 0 data
	Ch_t Ch[6];

	// LTA data - Ch1
	Ch_t LTA1;

	// Set flag to indicate Coil is present
	bool coilPresent;

	// Which sensor mode are we running now
	Setup_Type_e sensorMode;

} IQS621_t;


/**************************************************************************************************/
/*																								  */
/*									IQS621 memory adresses										  */
/*																								  */
/**************************************************************************************************/
#define I2C_ADDRESS               0x44

#define VERSION_INFO              0x00
#define SYSTEM_FLAGS              0x10
#define CHANNEL_DATA              0x20
#define LTA			                  0x30
#define PXS_SETTINGS              0x40
#define PXS_UI_SETTINGS           0x50
#define HYST_UI_SETTINGS          0x60
#define ALS_SENS_SETTINGS         0x70
#define ALS_UI_SETTINGS           0x80
#define HALL_SENS_SETTINGS        0x90
#define HALL_UI_SETTINGS      	  0xA0
#define DEV_SETTINGS 		          0xD0
#define DIRECT_ADDRESS	          0xF0
#define DIRECT_DATA	              0xF1


/**************************************************************************************************/
/*                                                  */
/*                  IQS620 memory adresses                      */
/*                                                  */
/**************************************************************************************************/

#define PXS_620_SETTINGS_0        0x40
#define PXS_620_SETTINGS_1        0x50
#define PXS_620_UI_SETTINGS       0x60
#define SAR_620_UI_SETTINGS       0x70
#define HYST_620_UI_SETTINGS      0x80
#define HALL_620_SENS_SETTINGS    0x90
#define HALL_620_UI_SETTINGS      0xA0
#define TEMP_620_UI_SETTINGS      0xC0
#define DEV_620_SETTINGS          0xD0



/* Bit definitions */
#define ACK_RESET				  0x40
#define REDO_ATI_ALL			0x02
#define DO_RESEED				  0x01


/* Setup Registers */

// PXS Settings 0 - 0x40
const static uint8_t PXS_Setup_0[] = {
		PXS_SETTINGS_0	,
		PXS_SETTINGS_1	,
		PXS_SETTINGS_2	,
		PXS_SETTINGS_3	,
		PXS_SETTINGS_4	,
		PXS_SETTINGS_5	,
		PXS_SETTINGS_6	,
		PXS_SETTINGS_7	,
		PXS_SETTINGS_8	,
		PXS_SETTINGS_9	,
		PXS_SETTINGS_10,
		PXS_SETTINGS_11,
		PXS_SETTINGS_12,
		PXS_SETTINGS_13,

};


// PXS UI - 0x50
const static uint8_t PXSUi[] = {
		PXS_UI_SETTINGS_0,
		PXS_UI_SETTINGS_1,
		PXS_UI_SETTINGS_2,
		PXS_UI_SETTINGS_3,
		PXS_UI_SETTINGS_4,
};


// Metal Detect UI - 0x60
const static uint8_t MetalDetect[] = {
		METAL_UI_SETTINGS_0,
		METAL_UI_SETTINGS_1,
		METAL_UI_SETTINGS_2,
		METAL_UI_SETTINGS_3
};

const static uint8_t Light_Sens[] = {
    LIGHT_SENSOR_SETTINGS_0,
    LIGHT_SENSOR_SETTINGS_1,
    LIGHT_SENSOR_SETTINGS_2,
    LIGHT_SENSOR_SETTINGS_3
};

const static uint8_t ALS_UI[] = {
    ALS_UI_SETTINGS_0,
    ALS_UI_SETTINGS_1,
    ALS_UI_SETTINGS_2,
    ALS_UI_SETTINGS_3
};


//LIGHT_SENSOR_SETTINGS_3

// HALL - 0x90
const static uint8_t Hall_Sens[] = {
		HALL_SENSOR_SETTINGS_0,
		HALL_SENSOR_SETTINGS_1,
		HALL_SENSOR_SETTINGS_2,
		HALL_SENSOR_SETTINGS_3
};

// HALL - 0xA0
const static uint8_t Hall_UI[] = {
		HALL_UI_SETTINGS_0,
		HALL_UI_SETTINGS_1,
		HALL_UI_SETTINGS_2
};

// Dev Setup - 0xD0
const static uint8_t DevSetup[] = {
		SYSTEM_SETTINGS|REDO_ATI_ALL|DO_RESEED, //|ACK_RESET,
		ACTIVE_CHS,
		PMU_SETTINGS,
		REPORT_RATES_TIMINGS_0,
		REPORT_RATES_TIMINGS_1,
		REPORT_RATES_TIMINGS_2,
		REPORT_RATES_TIMINGS_3
};



#endif /* __IQS62x_H__ */


