/******************************************************************************
 *                                                                             *
 *                                                                             *
 *                             Copyright by                                  *
 *                                                                             *
 *                            Azoteq (Pty) Ltd                                *
 *                        Republic of South Africa                            *
 *                                                                             *
 *                         Tel: +27(0)21 863 0033                             *
 *                        E-mail: info@azoteq.com                             *
 *                                                                             *
 *=============================================================================*
 * @file      IQS62x_PIR_Nano.ino                          *
 * @brief     Arduino example code for IQS62x Nano Shield                       *
 * @author    LC Theron - Azoteq PTY Ltd                             *
 * @version   V1.0.0                                                        *
 * @date      10/08/2018                                                   *
 *******************************************************************************/
 
#include "Arduino.h"
#include "src/I2C/I2C.h"
#include "src/Display/Display.h"
#include "IQS62x.h"
#include "IQS621_Init.h"
#include "Types.h"
#include <Wire.h>   
#include "IQS620A_Init.h"

// LED Defines
#define LED_1         3
#define LED_2         11
#define IR_LED        10

//EEPROM Defines
#define WP_E2         9
#define E2_Device_A   0x50
#define VREF          6

// Enum to determine what to show on screen
typedef enum IC_Type {
  IQS620,
  IQS620n,
  IQS621,
} IC_Type_e;


// What type of IC is this?
IC_Type_e ICType;

Timer_t Mode_Timer = {0};
Timer_t Sec_Timer = {0};
Timer_t Walk_Timer = {0};
Timer_t Walk_Timer_Activated = {0};

// IQS62x instance
I2C i2c(I2C_ADDRESS, IQS62x_RDY, Active_Low);
//EEPROM instance
I2C Ei2c(E2_Device_A, IQS62x_RDY);

IQS621_t iqs621;        // Create variable for iqs621
IQS620n_t iqs620;        // Create variable for iqs620 production version


//User Variables
int Leds[2] = {LED_1, LED_2};
// Buffer to read data into
uint8_t buffer[20];



uint8_t res;
bool refreshDisplay = false;
int E2_addr = 0;
uint8_t jumper_config = 0; //1 - DUAL PIR ; 2 - PIR and Touch ; 3 - Inductive (Determined in Setup)
uint8_t *E2_Buffer;
void setup()
{
  
  Serial.begin(115200);
  Serial.println("Setup started:");

  pinMode(VREF, OUTPUT); 
  pinMode(LED_1, OUTPUT); 
  pinMode(LED_2, OUTPUT); 
  digitalWrite(VREF, HIGH);
  pinMode(WP_E2, OUTPUT); 
  digitalWrite(WP_E2, LOW);
  delay(100);

 /*//Example to write some data to EEPROM
  buffer[0] = 0x11;
  buffer[1] = 0x12;
  buffer[2] = 0x13;
  write_EEPROM(0x00,3, buffer); 
 //Read some data from EEPROM
  E2_Buffer = read_EEPROM(0x00,3); 
*/

  while(!i2c.isDeviceReady());
  i2c.read(VERSION_INFO, 2, buffer, I2C_Stop);

  // Set the appropriate IC
  if(buffer[0] == IQS620_PRODUCT_NR){
    ICType = IQS620n;
    Serial.println("IQS620A");
    res = setup_iqs620(); 
    }
  else if(buffer[0] == IQS621_PRODUCT_NR){
    ICType = IQS621;
    Serial.println("IQS621");
    res = setup_iqs621();
    }
  else{
      Serial.println("Error - Wrong IC");
    }

  attachInterrupt(digitalPinToInterrupt(IQS62x_RDY), read_IQS62x, FALLING);
  Serial.println("Setup done.");

}

void loop()
{

  process_IQS62x_events();

}

/**************************************************************************************************/
/*                                                  */
/*                    Process IQS62x                       */
/*                                                  */
/**************************************************************************************************/


/**
   @brief  Process the Events that the IQS62x reported to us. 
   @param   none
   @retval  None
*/



void process_IQS62x_events()
{
  if(ICType == IQS620n){
      // Check that no reset has occurred
    if (iqs620.SystemFlags.ShowReset){
        // Re-intialise the IQS620
        setup_iqs620();

        Serial.println("Reset");
    }

  //Use LEDS to display activation
  digitalWrite(Leds[0], !(iqs620.PXSUIFlags.CH0_T_Out));
  digitalWrite(Leds[1], !(iqs620.PXSUIFlags.CH1_T_Out));
  }
  else if (ICType == IQS621){
        // Check that no reset has occurred
     if (iqs621.SystemFlags.ShowReset){
        // Re-intialise the IQS621
        setup_iqs621();

        Serial.println("Reset");
    }

  //Use LEDS to display activation
  digitalWrite(Leds[0], !(iqs621.PXSUIFlags.CH0_T_Out));
  digitalWrite(Leds[1], !(iqs621.PXSUIFlags.CH1_T_Out));
  }


}

/**************************************************************************************************/
/*                                                  */
/*                    Read IQS62x                       */
/*                                                  */
/**************************************************************************************************/

/**
   @brief Read IQS62x flags and channel data and write to serial monitor
   @param   None
   @retval  None
*/


void read_IQS62x(void)
{
    if(ICType == IQS620n){
      //Read the system Flags
      res = i2c.read(SYSTEM_FLAGS, 9, &iqs620.SystemFlags.SystemFlags, I2C_Repeat_Start);
      // Read PIR Channels Data - 4 bytes
      res |= i2c.read(CHANNEL_DATA, 6, &iqs620.Ch[0].Ch_Low, I2C_Stop);

      //Print channel data to serial monitor
      if(jumper_config !=3){
        Serial.print(iqs620.Ch[0].Ch);
        Serial.print("\t");
        Serial.println(iqs620.Ch[1].Ch);

      }
      else{
        Serial.println(iqs620.Ch[2].Ch);
      }
    }
    else if (ICType == IQS621){
      //Read the system Flags
      res = i2c.read(SYSTEM_FLAGS, 9, &iqs621.SystemFlags.SystemFlags, I2C_Repeat_Start);
      // Read PIR Channels Data - 4 bytes
      res |= i2c.read(CHANNEL_DATA, 4, &iqs621.Ch[0].Ch_Low, I2C_Stop);
  
      //Print channel data to serial monitor
      Serial.print(iqs621.ALSValue.ALSValue);
      Serial.print("\t");
      if(jumper_config !=3){
      Serial.print(iqs621.Ch[0].Ch);
      Serial.print("\t");
      }
      Serial.println(iqs621.Ch[1].Ch);
    }

}

/**************************************************************************************************/
/*                                                  */
/*                    Setup IQS621                        */
/*                                                  */
/**************************************************************************************************/

/**
   @brief Check which setup we need to run for the IQS621 - capacitive or inductive or PIR
   @param
   @retval  None
*/
uint8_t setup_iqs621()
{
  uint8_t res = 0;

  //Determine Configuration (PIR/Touc/Inductive)
  while(!i2c.isDeviceReady());

  // Fill buffer with first settings
  buffer[0] = 0x11;
  buffer[1] = 0x12;
  buffer[2] = 0x07;
  buffer[3] = 0x47;
  buffer[4] = 0x20;
  buffer[5] = 0x20;
  buffer[6] = 0x26;
  buffer[7] = 0x06;
  buffer[8] = 0x20;
  buffer[9] = 0x01;
  buffer[10] = 0x62;
  buffer[11] = 0x58;
  buffer[12] = 0x01;
  buffer[13] = 0x01;

  res |= i2c.write(PXS_SETTINGS, 13, buffer, I2C_Repeat_Start);

  buffer[0] = SYSTEM_SETTINGS | 0x42;

  res |= i2c.write(DEV_SETTINGS, 1, buffer, I2C_Stop);

  // Wait for Redo Ati to complete
  do {
    delay(10);
    res |= i2c.read(SYSTEM_FLAGS, 1, &iqs621.SystemFlags.SystemFlags, I2C_Stop);

  } while (!res && iqs621.SystemFlags.InAti);
  
  delay(10);
  read_IQS62x();
  delay(10);
  read_IQS62x();


  if(iqs621.Ch[1].Ch >2000){
    if(iqs621.Ch[0].Ch >2000){
      Serial.println(iqs621.Ch[0].Ch);
      Serial.println("Inductive Mode");
      jumper_config = 3;
    }
    else{
      Serial.println(iqs621.Ch[1].Ch);
      Serial.println("PIR and touch Mode");
      jumper_config = 2;
    }
  }
  else
  {
     Serial.println(iqs621.Ch[1].Ch);
     Serial.println("Dual PIR mode");
     jumper_config = 1;
  }



  while(!i2c.isDeviceReady());

  // Write Configuration Settings
  buffer[0] = PXS_SETTINGS_0;
  
  if(jumper_config == 1){
    buffer[1] = PXS_SETTINGS_1;
  }
  if(jumper_config == 2){
    buffer[1] = 0x02;
  }
  if(jumper_config == 3){
    buffer[1] = 0xAB;
  }

  if(jumper_config == 1){
    buffer[2] = PXS_SETTINGS_2;
  }
  if(jumper_config == 2){
    buffer[2] = PXS_SETTINGS_2;
  }
  if(jumper_config == 3){
    buffer[2] = 0x77;
  }
  
  buffer[3] = PXS_SETTINGS_3;
  buffer[4] = PXS_SETTINGS_4;
  
  if(jumper_config == 1){
    buffer[5] = PXS_SETTINGS_5;
  }
  if(jumper_config == 2){
    buffer[5] = 0xD0;
  }
  if(jumper_config == 3){
    buffer[5] = 0xD0;
  }

  buffer[6] = PXS_SETTINGS_6;
  buffer[7] = PXS_SETTINGS_7;
  
  if(jumper_config == 1){
    buffer[8] = PXS_SETTINGS_8;
  }
  if(jumper_config == 2){
    buffer[8] = PXS_SETTINGS_8;
  }
  if(jumper_config == 3){
    buffer[8] = 0x62;
  }

  buffer[9] = PXS_SETTINGS_9;
  buffer[10] = PXS_SETTINGS_10;
  buffer[11] = PXS_SETTINGS_11;
  buffer[12] = PXS_SETTINGS_12;
  buffer[13] = PXS_SETTINGS_13;
  
  res |= i2c.write(PXS_SETTINGS, 14, buffer, I2C_Repeat_Start);

  buffer[0] = PXS_UI_SETTINGS_0; // Prox
  buffer[1] = PXS_UI_SETTINGS_1; // Touch
  buffer[2] = PXS_UI_SETTINGS_2; // Prox
  if(jumper_config == 1){
    buffer[3] = PXS_UI_SETTINGS_3; 
  }
  if(jumper_config == 2){
    buffer[3] = 0x34;
  }
  if(jumper_config == 3){
    buffer[3] = 0x34; 
  }

  buffer[4] = PXS_UI_SETTINGS_4;

  res |= i2c.write(PXS_UI_SETTINGS, 5, buffer, I2C_Repeat_Start);

  buffer[0] = LIGHT_SENSOR_SETTINGS_0; 
  buffer[1] = LIGHT_SENSOR_SETTINGS_1; 
  buffer[2] = LIGHT_SENSOR_SETTINGS_2; 
  buffer[3] = LIGHT_SENSOR_SETTINGS_3; 

  res |= i2c.write(ALS_SENS_SETTINGS, 4, buffer, I2C_Repeat_Start);

  buffer[0] = ALS_UI_SETTINGS_0;
  buffer[1] = ALS_UI_SETTINGS_1; 
  buffer[2] = ALS_UI_SETTINGS_2; 
  buffer[3] = ALS_UI_SETTINGS_3; 

  res |= i2c.write(ALS_UI_SETTINGS, 4, buffer, I2C_Repeat_Start);


  // Fill buffer with first settings
  buffer[0] = SYSTEM_SETTINGS;
  if(jumper_config == 1){
    buffer[1] = ACTIVE_CHS;
  }
  if(jumper_config == 2){
    buffer[1] = ACTIVE_CHS;
  }
  if(jumper_config == 3){
    buffer[1] = 0x7E;
  }

  buffer[2] = PMU_SETTINGS;
  buffer[3] = REPORT_RATES_TIMINGS_0;
  buffer[4] = REPORT_RATES_TIMINGS_1;
  buffer[5] = REPORT_RATES_TIMINGS_2;
  buffer[6] = REPORT_RATES_TIMINGS_3;
  buffer[7] = GLOBAL_EVENT_MASK;

  res |= i2c.write(DEV_SETTINGS, 8, buffer, I2C_Repeat_Start);

  buffer[0] = SYSTEM_SETTINGS | 0x42;

  res |= i2c.write(DEV_SETTINGS, 1, buffer, I2C_Stop);

  // Wait for Redo Ati to complete
  do {

    res |= i2c.read(SYSTEM_FLAGS, 1, &iqs621.SystemFlags.SystemFlags, I2C_Stop);

  } while (!res && iqs621.SystemFlags.InAti);

  return res;

}

/**************************************************************************************************/
/*                                                  */
/*                    Setup IQS620                       */
/*                                                  */
/**************************************************************************************************/

/**
   @brief Check which setup we need to run for the IQS620 - capacitive or inductive or PIR
   @param
   @retval  None
*/
uint8_t setup_iqs620()
{
  uint8_t res = 0;

   //Determine Configuration (PIR/Touc/Inductive)
  while(!i2c.isDeviceReady());

  // Fill buffer with first settings
  buffer[0] = 0x11;
  buffer[1] = 0x12;
  buffer[2] = 0x12;
  buffer[3] = 0x4F;
  buffer[4] = 0x4F;
  buffer[5] = 0x0F;
  buffer[6] = 0x60;
  buffer[7] = 0x60;
  buffer[8] = 0x50;
  buffer[9] = 0x06;
  buffer[10] = 0x06;
  buffer[11] = 0x06;


  res |= i2c.write(PXS_620_SETTINGS_0, 12, buffer, I2C_Repeat_Start);

  buffer[0] = SYSTEM_620_SETTINGS | 0x42;

  res |= i2c.write(DEV_620_SETTINGS, 1, buffer, I2C_Stop);

  // Wait for Redo Ati to complete
  do {

    delay(10);
    res |= i2c.read(SYSTEM_FLAGS, 1, &iqs620.SystemFlags.SystemFlags, I2C_Stop);

  } while (!res && iqs620.SystemFlags.InAti);
  
  delay(10);
  read_IQS62x();
  delay(10);
  read_IQS62x();

  if(iqs620.Ch[1].Ch >2000){
    if(iqs620.Ch[0].Ch >2000){
      Serial.println(iqs620.Ch[0].Ch);
      Serial.println("Inductive Mode");
      jumper_config = 3;
    }
    else{
      Serial.println(iqs620.Ch[1].Ch);
      Serial.println("PIR and touch Mode");
      jumper_config = 2;
    }
  }
  else{
      Serial.println(iqs620.Ch[1].Ch);
      Serial.println("Dual PIR mode");
      jumper_config = 1;
  }


  while(!i2c.isDeviceReady());

  buffer[0] = PXS_620_SETTINGS0_0;
  
  if(jumper_config == 1){
    buffer[1] = PXS_620_SETTINGS0_1;
  }
  if(jumper_config == 2){
    buffer[1] = 0x02;
  }
  if(jumper_config == 3){
    buffer[1] = 0x02;
  }

  if(jumper_config == 1){
    buffer[2] = PXS_620_SETTINGS0_2;
  }
  if(jumper_config == 2){
    buffer[2] = 0x30;
  }
  if(jumper_config == 3){
    buffer[2] = 0xB3;
  }

  buffer[3] = PXS_620_SETTINGS0_3;
  buffer[4] = PXS_620_SETTINGS0_4;
  buffer[5] = PXS_620_SETTINGS0_5;
  buffer[6] = PXS_620_SETTINGS0_6;
  
  if(jumper_config == 1){
    buffer[7] = PXS_620_SETTINGS0_7;
  }
  if(jumper_config == 2){
    buffer[7] = 0xD0;
  }
  if(jumper_config == 3){
    buffer[7] = 0xD0;
  }

  if(jumper_config == 1){
    buffer[8] = PXS_620_SETTINGS0_8;
  }
  if(jumper_config == 2){
    buffer[8] = PXS_620_SETTINGS0_8;
  }
  if(jumper_config == 3){
    buffer[8] = 0x55;
  }
  
  buffer[9] = PXS_620_SETTINGS0_9;
  buffer[10] = PXS_620_SETTINGS0_10;
  buffer[11] = PXS_620_SETTINGS0_11;

  res |= i2c.write(PXS_620_SETTINGS_0, 12, buffer, I2C_Repeat_Start);

  if(jumper_config == 1){
    buffer[0] = PXS_620_SETTINGS1_0;
  }
  if(jumper_config == 2){
    buffer[0] = PXS_620_SETTINGS1_0;
  }
  if(jumper_config == 3){
    buffer[0] = 0x61;
  }

  res |= i2c.write(PXS_620_SETTINGS_1, 1, buffer, I2C_Repeat_Start);
  
  buffer[0] = PXS_620_UI_SETTINGS_0; 
  buffer[1] = PXS_620_UI_SETTINGS_1; 
  buffer[2] = PXS_620_UI_SETTINGS_2; 
  buffer[3] = PXS_620_UI_SETTINGS_3; 
  buffer[4] = PXS_620_UI_SETTINGS_4; 
  buffer[5] = PXS_620_UI_SETTINGS_5; 
 
  if(jumper_config == 1){
    buffer[6] = PXS_620_UI_SETTINGS_6; // Touch
  }
  if(jumper_config == 2){
    buffer[6] = PXS_620_UI_SETTINGS_6; // Touch
  }
  if(jumper_config == 3){
    buffer[3] = 0x3C; // Touch
  }

  res |= i2c.write(PXS_620_UI_SETTINGS, 5, buffer, I2C_Repeat_Start);

  buffer[0] = SYSTEM_620_SETTINGS;
  if(jumper_config == 1){
    buffer[1] = ACTIVE_620_CHS;
  }
  if(jumper_config == 2){
    buffer[1] = ACTIVE_620_CHS;
  }
  if(jumper_config == 3){
    buffer[1] = 0x3C;
  }

  buffer[2] = PMU_620_SETTINGS;
  buffer[3] = REPORT_620_RATES_TIMINGS_0;
  buffer[4] = REPORT_620_RATES_TIMINGS_1;
  buffer[5] = REPORT_620_RATES_TIMINGS_2;
  buffer[6] = REPORT_620_RATES_TIMINGS_3;

  res |= i2c.write(DEV_620_SETTINGS, 8, buffer, I2C_Repeat_Start);

  buffer[0] = SYSTEM_SETTINGS | 0x42;

  res |= i2c.write(DEV_620_SETTINGS, 1, buffer, I2C_Stop);

  // Wait for Redo Ati to complete
  do {
    delay(40);
    res |= i2c.read(SYSTEM_FLAGS, 1, &iqs620.SystemFlags.SystemFlags, I2C_Stop);

  } while (!res && iqs620.SystemFlags.InAti);

  return res;

}

/**************************************************************************************************/
/*                                                  */
/*                    Read EEPROM                     */
/*                                                  */
/**************************************************************************************************/
/**
   @brief Read the EEPROM on the PIR Nano Shield
   @param Address on the EEPROM, number of bytes
   @retval  Value(s) read
   */
uint8_t*  read_EEPROM(uint8_t E_Address, uint8_t nr_of_bytes)
{
  uint8_t E_buffer[nr_of_bytes];
  
  delay(5);
  while(!Ei2c.isDeviceReady());
  Ei2c.read(E_Address, nr_of_bytes, &E_buffer[0], I2C_Stop);
  delay(5); 
  
  return E_buffer;
  
}

  /**************************************************************************************************/
/*                                                  */
/*                    Write EEPROM                     */
/*                                                  */
/**************************************************************************************************/
/**
   @brief Write the EEPROM on the PIR Nano Shield
   @param Address on the EEPROM, number of byte to write, array of bytes to write
   @retval  none
   */
void  write_EEPROM(uint8_t E_Address,uint8_t nr_of_bytes, uint8_t* E_buffer)
{
  delay(5);
  while(!Ei2c.isDeviceReady());
  Ei2c.write(E_Address, nr_of_bytes, E_buffer, I2C_Stop);
  delay(5); 

}

/**************************************************************************************************/
/*                                                  */
/*                    Timer Functions                       */
/*                                                  */
/**************************************************************************************************/

/**
   @brief
*/
void setTimer(Timer_t* timer)
{
  timer->TimerExpired = false;
  timer->Timer_start = millis();  // get this instant millis
}

/**
   @brief
*/
void setTimer(Timer_t* timer, uint32_t time)
{
  timer->TimerExpired = false;
  timer->Timer_start = millis();  // get this instant millis
  timer->Timer_counter = time;  // the timeout time for the timer
}

uint16_t currentTime(Timer_t* timer)
{
  // This is a timeout
  return (timer->Timer_counter - (millis() - timer->Timer_start));

}

bool timerExpired(Timer_t* timer)
{
  // This is a timeout
  if ((millis() - timer->Timer_start) >= timer->Timer_counter)
    timer->TimerExpired = true;
  // We haven't timed out yet
  else
    timer->TimerExpired  = false;

  // Return the state of this timer
  return timer->TimerExpired;
}

/**************************************************************************************************/
/*                                                  */
/*                    EEPROM Functions                       */
/*                                                  */
/**************************************************************************************************/

/**
   @brief
*/
   
void writeEEPROM(int deviceaddress, unsigned int eeaddress, byte data ) 
{
  Wire.beginTransmission(deviceaddress);
  Wire.write((int)(eeaddress >> 8));   // MSB
  Wire.write((int)(eeaddress & 0xFF)); // LSB
  Wire.write(data);
  Wire.endTransmission();
 
  delay(5);
}
 
byte readEEPROM(int deviceaddress, unsigned int eeaddress ) 
{
  byte rdata = 0xFF;
 
  Wire.beginTransmission(deviceaddress);
  Wire.write((int)(eeaddress >> 8));   // MSB
  Wire.write((int)(eeaddress & 0xFF)); // LSB
  Wire.endTransmission();
 
  Wire.requestFrom(deviceaddress,1);
 
  if (Wire.available()) rdata = Wire.read();
 
  return rdata;
}
